@@ -1,16 +1,36 @@
 ## -*- truncate-lines: t; -*-
-## Copyright (C) 2008-20  Enrico Schumann
+## Copyright (C) 2008-24  Enrico Schumann
 
-is_valid_ISIN <- function(isin) {
+.ISIN <- "[^A-Za-z]*([A-Za-z][A-Za-z][A-Za-z0-9]{9,9}[0-9])[^0-9]*"
+
+is_valid_ISIN <- function(isin, NA.FALSE = FALSE) {
 
     if (as.character(sys.call(sys.parent()))[1L] == "isValidISIN")
         .Deprecated("is_valid_ISIN")
 
-    validNC <- 12L
     result <- logical(length(isin))
-    isin <- isin[hasValidNC <- which(nchar(isin) == validNC)]
+    names(result) <- isin
+
+    ## any NA?
+    i.NA <- is.na(isin)
+    good <- !i.NA
+    ## ==> now, everything that is not NA is considered good
+
+    ## number of chars == 12?
+    validNC <- 12L
+    good[good] <- good[good] &
+                  nchar(isin[good]) == validNC
+
+    ## check digits %in% 0:9?
+    cd <- substr(isin[good], validNC, validNC)
+    good[good] <- good[good] &
+                  cd %in% c("0", "1", "2", "3", "4",
+                            "5", "6", "7", "8", "9")
+
+    isin <- isin[good]
     if (!length(isin))
         return(result)
+
     isin <- toupper(isin)
     pasteAndSplit <- function(s)
         as.numeric(unlist(strsplit(paste(s, collapse = ""), "")))
@@ -21,23 +41,21 @@ is_valid_ISIN <- function(isin) {
         x
     }
     cd <- as.integer(substr(isin, validNC, validNC))  ## control digit
-    v0 <- substr(isin, 1L, validNC - 1L) ## v0 is vector
+    v0 <- substr(isin, 1L, validNC - 1L)              ## v0 is vector
     v0 <- strsplit(v0, "")  ## get list of char vecs; v0 is list
     v0 <- lapply(v0, function(x) match(x, c(as.character(0:9), LETTERS)) - 1L)
     v0 <- lapply(v0, pasteAndSplit)
     v0 <- lapply(v0, multSecby2)
     v0 <- lapply(v0, pasteAndSplit)
 
-    result[hasValidNC] <- (10L - sapply(v0, sum) %% 10L) %% 10L == cd
+    result[good] <- (10L - sapply(v0, sum) %% 10L) %% 10L == cd
+    result[i.NA] <- if (NA.FALSE) FALSE else NA
     result
 }
 
-.ISIN <- "[^A-Za-z]*([A-Za-z][A-Za-z][A-Za-z0-9]{9,9}[0-9])[^0-9]*"
-
-
 is_valid_SEDOL <- function(SEDOL, NA.FALSE = FALSE) {
 
-    ## ## TODO strip hyphens? B-1H5-4P6
+    ## TODO strip hyphens, as in B-1H5-4P6 ?
 
     i.NA <- is.na(SEDOL)
     SEDOL[i.NA] <- "A"
