@@ -233,3 +233,49 @@ rc <- function(R, weights, timestamp, segments = NULL,
     attr(total, "adjusted") <- C.adj
     total
 }
+
+
+## A. Colin -- A Brinson Model Alternative: an Equity
+## Attribution Model with Orthogonal Risk Contributions, 2007
+.colin2007 <- function(weights, weights.bm, R) {
+
+    if (is.null(names(weights)) &&
+        is.null(names(weights.bm)) &&
+        is.null(names(R)))
+        names(weights) <- names(weights.bm)  <- names(R) <- 1:length(R)
+
+
+    n.w <- names(weights)
+    n.bm <- names(weights.bm)
+    n <- sort(unique(c(n.w, n.bm)))
+    w_ <- w <- W <- numeric(length(n))
+    names(w_) <- names(w) <- names(W) <- n
+    r <- R[n]
+    w[n.w] <- weights
+    W[n.bm] <- weights.bm
+    w_[w != 0 & W != 0] <- W[w != 0 & W != 0]
+    w_ <- w_/sum(w_)
+
+    df <- data.frame(portfolio = w*r,
+                     benchmark = W*r,
+                     selection = (w_ - W)*r,
+                     allocation = (w - w_)*r)
+    df <- rbind(df, apply(df, 2, sum))
+    row.names(df)[nrow(df)] <- "total"
+    df
+    ## c(portfolio = sum(w*r),
+    ##   benchmark = sum(W*r),
+    ##   selection = sum((w_ - W)*r),
+    ##   allocation = sum((w - w_)*r))
+
+}
+
+## weights <- c(a=0.5,b=0.1,cash=0.4)
+## weights.bm <- c(a=0.5,b=0.5)
+## R <- c(a=0.02,b=-0.01, cash = 0)
+## .colin(weights, weights.bm, R)
+
+## weights <- c(0.8,0,0.2,0,0)
+## weights.bm <- c(.3,.3,.1,.1,.2)
+## R <- c(2,-2,1,-2,0)/100
+## dput(.colin(weights, weights.bm, R))
